import debounce from 'lodash.debounce'
import { getMapConfig, setServiceHost } from './mapConfig.js'

function createInfoWindowContent (content, {
  color = '#000000',
  fontSize = 14,
  borderRadius = 3,
  borderWidth = 1,
  borderColor = '#000000',
  bgColor = '#ffffff',
  padding = 10,
  textAlign = 'center',
  anchorX = 0,
  anchorY = 0
}, mapInstance) {
  const root = document.createElement('div')
  const close = document.createElement('div')
  root.style.color = color
  root.style['font-size'] = fontSize + 'px'
  root.style.position = 'relative'
  root.style.border = '1px solid #eeeeee'
  root.style.borderRadius = borderRadius + 'px'
  root.style['border-width'] = borderWidth + 'px'
  root.style['border-color'] = borderColor
  root.style['background-color'] = bgColor
  root.style.padding = padding + 'px'
  root.style['text-align'] = textAlign
  root.style.top = anchorY + 'px'
  root.style.left = anchorX + 'px'
  close.innerText = '+'
  close.style.transform = 'rotate(45deg)'
  close.style.position = 'absolute'
  close.style.top = '0'
  close.style.right = '0'
  close.style['font-size'] = '20px'
  close.style.width = '15px'
  close.style.height = '15px'
  close.style['line-height'] = '15px'
  root.appendChild(close)
  root.appendChild(document.createTextNode(content))
  close.onclick = () => {
    mapInstance.clearInfoWindow()
  }
  return root
}

export default class MapInstance {
  constructor (event, params, callbackId, invokeCallback, emitCustomEvent, jsBridge) {
    this.mapdata = Object.assign(params, (params.setting || {}))
    this.markersInstence = {}
    this.initCenterOffset = []
    this.initRegonData = {}
    this.makerCluster = null
    this.invokeCallback = invokeCallback
    this.emitCustomEvent = emitCustomEvent
    this.FinChatJSBridge = jsBridge
    this.prepareLoader()
    this.loadScript(event, callbackId)
  }

  
  prepareLoader() {
    const { serviceHost } = getMapConfig()
    setServiceHost(serviceHost)
  }

  loadScript(event, callbackId) {
    let script 
    if(!document.querySelector('#amap-loader')) {
      script = document.createElement("script");
      script.type = "text/javascript";
      script.src = "https://webapi.amap.com/loader.js";
      document.body.appendChild(script);
      script.setAttribute("id", 'amap-loader');
      script.onload = () => {
        this.amapload (event, callbackId)
      }
    } else {
      setTimeout(() => {
        this.amapload(event, callbackId)
      }, 500);
    }
  }
  amapload(event, callbackId) {
    const { mapJSAPIKey } = getMapConfig()
    AMapLoader.load({
      key: mapJSAPIKey
    }).then((AMap) => {
      this.mapInit(AMap)
      this.invokeCallback('ok', event, callbackId, {})
    }).catch(e => {
      throw new Error(e)
    })
  }

  mapInit (AMap) {
    this.mapConstructor = AMap
    const {
      cid,
      scale,
      minScale,
      maxScale,
      longitude,
      latitude,
      skew,
      enable3D,
      showScale,
      enableZoom,
      enableScroll,
      enableRotate,
      enableSatellite,
      enableTraffic
    } = this.mapdata
    const inner = document.querySelectorAll(`[nativeid="${cid}"]`)
    this.inner = inner[0]
    this.mapInstance = new AMap.Map(inner[0], {
      zoom: scale,
      zooms: [minScale || 3, maxScale || 20],
      center: [longitude, latitude],
      pitch: skew,
      showBuildingBlock: enable3D,
      zoomEnable: enableZoom,
      scrollWheel: enableScroll,
      rotateEnable: enableRotate,
      dragEnable: true,
      viewMode: '3D',
      expandZoomRange: true
    })
    this.addEvent(AMap)
    this.initInfoWindow(AMap)
    this.initAttr()
    this.initLayer(AMap, enableSatellite, enableTraffic)
    this.initPlugins(AMap, showScale)
  }

  addEvent (AMap) {
    AMap.event.addListener(this.mapInstance, 'click', debounce((e) => {
      const longitude = e.lnglat.getLng()
      const latitude = e.lnglat.getLat()
      this.emitCustomEvent({
        event: 'custom_event_onMapTask',
        params: {
          eventName: 'tap',
          mapId: this.mapdata.mapId,
          longitude,
          latitude
        }
      })
    }, 100))

    AMap.event.addListener(this.mapInstance, 'movestart', (e) => {
      const changekey = ['longitude', 'latitude', 'moveToLocation'].find(key => this[`${key}ChangeFlag`])
      if (changekey) {
        this.emitCustomEvent({
          event: 'custom_event_onMapTask',
          params: {
            eventName: 'regionchange',
            mapId: this.mapdata.mapId,
            causedBy: 'update',
            type: 'begin'
          }
        })
      }
    })
    AMap.event.addListener(this.mapInstance, 'moveend', (e) => {
      const changekey = ['longitude', 'latitude', 'moveToLocation'].find(key => this[`${key}ChangeFlag`])
      if (changekey) {
        const mapdata = this.getMapRegonData()
        this[`${changekey}ChangeFlag`] = false
        this.emitCustomEvent({
          event: 'custom_event_onMapTask',
          params: {
            eventName: 'regionchange',
            mapId: this.mapdata.mapId,
            causedBy: 'update',
            type: 'end',
            ...mapdata
          }
        })
      }
    })
    AMap.event.addListener(this.mapInstance, 'zoomstart', (e) => {
      this.emitCustomEvent({
        event: 'custom_event_onMapTask',
        params: {
          eventName: 'regionchange',
          mapId: this.mapdata.mapId,
          causedBy: e.target ? 'gesture' : 'update',
          type: 'begin'
        }
      })
    })
    AMap.event.addListener(this.mapInstance, 'zoomend', (e) => {
      const mapdata = this.getMapRegonData()
      this.initRegonData = mapdata.region
      this.emitCustomEvent({
        event: 'custom_event_onMapTask',
        params: {
          eventName: 'regionchange',
          mapId: this.mapdata.mapId,
          causedBy: e.target ? 'scroll' : 'update',
          type: 'end',
          ...mapdata
        }
      })
    })
    AMap.event.addListener(this.mapInstance, 'dragstart', (e) => {
      this.emitCustomEvent({
        event: 'custom_event_onMapTask',
        params: {
          eventName: 'regionchange',
          mapId: this.mapdata.mapId,
          causedBy: 'gesture',
          type: 'begin'
        }
      })
    })
    AMap.event.addListener(this.mapInstance, 'dragend', (e) => {
      const mapdata = this.getMapRegonData()
      this.initRegonData = mapdata.region
      this.emitCustomEvent({
        event: 'custom_event_onMapTask',
        params: {
          eventName: 'regionchange',
          mapId: this.mapdata.mapId,
          causedBy: 'drag',
          type: 'end',
          ...mapdata
        }
      })
    })
  }

  initInfoWindow (AMap) {
    this.infoWindow = new AMap.InfoWindow({
      isCustom: true,
      offset: new AMap.Pixel(0, -30)
    })
  }

  initAttr () {
    this.markersChanged(this.mapdata.markers)
    this.polylineChanged(this.mapdata.polyline)
    this.circlesChanged(this.mapdata.circles)
    this.controlsChanged(this.mapdata.controls)
    this.polygonsChanged(this.mapdata.polygons)
    this.rotateChanged(this.mapdata.rotate)
    this.enablePoiChanged(this.mapdata.enablePoiChanged)
    this.initRegonData = this.getMapRegonData().region
  }

  markersChanged (markers = [], clear) {
    if (clear) {
      this._removeMarkers(Object.keys(this.markersInstence))
    }
    markers.forEach(item => {
      const position = new this.mapConstructor.LngLat(item.longitude, item.latitude)
      const marker = new this.mapConstructor.Marker({
        position,
        anchor: 'bottom-center',
        offset: new this.mapConstructor.Pixel(0, 0),
        label: {
          content: item.label && item.label.content,
          direction: 'bottom'
        },
        icon: (item.iconPath && new this.mapConstructor.Icon({
          image: item.iconPath,
          size: new this.mapConstructor.Size(item.width || 36, item.height || 36),
          imageSize: new this.mapConstructor.Size(item.width || 36, item.height || 36)
        }))
      })
      marker.id = item.id || new Date().getTime()
      marker.id in this.markersInstence && (this._removeMarkers([marker.id]))
      marker.joinCluster = item.joinCluster || false
      const markerlistener = this.mapConstructor.event.addListener(marker, 'click', debounce((event) => {
        this.emitCustomEvent({
          event: 'custom_event_onMapTask',
          params: {
            eventName: 'markertap',
            mapId: this.mapdata.mapId,
            markerId: marker.id
          }
        })
        let popp
        if (item.callout) {
          popp = createInfoWindowContent(item.callout.content, item.callout, this.mapInstance)
        } else if (item.title) {
          popp = createInfoWindowContent(item.title, {}, this.mapInstance)
        }
        if (popp) {
          this.infoWindow.setContent(popp)
          this.infoWindow.open(this.mapInstance, position)
        }
      }, 100))
      marker.markerlistener = markerlistener
      if (marker.joinCluster) {
        this.makerCluster && this.makerCluster.addMarker(marker)        
      }

      this.mapInstance.add(marker)
      this.markersInstence[marker.id] = marker
    })
  }

  _removeMarkers (markerIds) {
    markerIds.forEach(mid => {
      const marker = this.markersInstence[mid]
      if (marker) {
        this.mapConstructor.event.removeListener(marker.markerlistener)
        this.mapInstance.remove(marker)
        this.makerCluster.removeMarker(marker)
        delete this.markersInstence[mid]
      }
    })
  }

  polylineChanged (val) {
    (this.polylines || []).forEach(e => {
      e.setMap(null)
    })
    if (val) {
      this.polylines = val.map(line => {
        const polyline = new this.mapConstructor.Polyline({
          path: (line.points || []).map(e => [e.longitude, e.latitude]),
          isOutline: line.borderWidth,
          outlineColor: line.borderColor,
          borderWeight: line.borderWidth,
          strokeColor: line.color || '#000000',
          strokeWeight: line.width || 1,
          strokeStyle: line.dottedLine ? 'dashed' : 'solid',
          strokeDasharray: [6, 4, 6]
        })
        polyline.setMap(this.mapInstance)
        return polyline
      })
    }
  }

  circlesChanged (val) {
    (this.circles || []).forEach(e => {
      e.setMap(null)
    })
    if (val) {
      this.circles = val.map(item => {
        const circle = new this.mapConstructor.Circle({
          center: [item.longitude, item.latitude],
          radius: item.radius || 0,
          strokeColor: item.color || '#000000',
          strokeWeight: item.strokeWidth || 1,
          fillColor: item.fillColor,
        })
        circle.setMap(this.mapInstance)
        return circle
      })
    }
  }

  controlsChanged (val) {
    (this.controls || []).forEach(e => {
      this.inner.removeChild(e)
    })
    const controls = []
    val.forEach(e => {
      if (e.iconPath) {
        e.position = Object.assign({
          top: 0, left: 0, width: 0, height: 0
        }, (e.position || {}))
        const div = document.createElement('div')
        const img = document.createElement('img')
        div.style = `position: absolute;z-index: 4;
                    top: ${(e.position.top * 1) + 'px'};
                    left: ${(e.position.left * 1) + 'px'};
                    width: ${e.position.width ? (e.position.width + 'px') : 'auto'};
                    height: ${e.position.height ? (e.position.height + 'px') : 'auto'};
                    `
        img.src = e.iconPath
        img.style = 'width:100%;height:100%'
        div.appendChild(img);
        (e.clickable && e.id !== undefined) && div.addEventListener('click', () => {
          this.emitCustomEvent({
            event: 'custom_event_onMapTask',
            params: {
              eventName: 'controltap',
              mapId: this.mapdata.mapId,
              controlId: e.id
            }
          })
        })
        controls.push(div)
        this.inner.appendChild(div)
      }
    })
    this.controls = controls
  }

  polygonsChanged (val) {
    (this.polygons || []).forEach(e => {
      e.setMap(null)
    })
    if (val) {
      this.polygons = val.map(item => {
        const polygon = new this.mapConstructor.Polygon({
          path: item.points.map(e => [e.longitude, e.latitude]),
          strokeColor: item.strokeColor || '#000000',
          strokeWeight: item.strokeWidth || 1,
          fillColor: item.fillColor,
          zIndex: item.zIndex,
        })
        polygon.setMap(this.mapInstance)
        return polygon
      })
    }
  }

  rotateChanged (val) {
    this.emitCustomEvent({
      event: 'custom_event_onMapTask',
      params: {
        eventName: 'regionchange',
        mapId: this.mapdata.mapId,
        causedBy: 'update',
        type: 'begin'
      }
    })
    this.mapdata.rotate = val
    this.mapInstance.setRotation(val)
    const mapdata = this.getMapRegonData()
    this.emitCustomEvent({
      event: 'custom_event_onMapTask',
      params: {
        eventName: 'regionchange',
        mapId: this.mapdata.mapId,
        causedBy: 'update',
        type: 'end',
        ...mapdata
      }
    })
  }

  enablePoiChanged (val) {
    this.mapdata.enablePoi = (val === undefined || val === true)
    this.mapInstance.setFeatures(this.countFeatures())
  }

  countFeatures () {
    const res = ['bg', 'road']
    this.mapdata.enablePoi && res.push('point')
    this.mapdata.enableBuilding && res.push('building')
    return res
  }

  initLayer (AMap, enableSatellite, enableTraffic) {
    this.satelliteLayer = new AMap.TileLayer.Satellite()
    enableSatellite && this.mapInstance.add(this.satelliteLayer)
    this.trafficLayer = new AMap.TileLayer.Traffic()
    enableTraffic && this.mapInstance.add(this.trafficLayer)
  }
  initPlugins (AMap, showScale) {
    this.mapInstance.plugin(['AMap.Scale', 'AMap.MarkerClusterer'], () => {
      this.scale = new AMap.Scale({
        visible: false
      })
      
      this.makerCluster = new AMap.MarkerClusterer(this.mapInstance, [], {
        zoomOnClick: true,
        gridSize: 60,
        maxZoom: 15,
        averageCenter: true
      }) 

      this.mapInstance.addControl(this.scale)
      if (showScale) {
        this.scale.show()
      } else {
        this.scale.hide()
      }
    })
  }

  getMapRegonData () {
    const center = this.mapInstance.getCenter()
    const bounds = this.mapInstance.getBounds().bounds
    return {
      centerLocation: {
        latitude: center.getLat(),
        longitude: center.getLng()
      },
      region: {
        northwest: {
          latitude: bounds[0].getLat(),
          longitude: bounds[0].getLng()
        },
        northeast: {
          latitude: bounds[1].getLat(),
          longitude: bounds[1].getLng()
        },
        southeast: {
          latitude: bounds[2].getLat(),
          longitude: bounds[2].getLng()
        },
        southwest: {
          latitude: bounds[3].getLat(),
          longitude: bounds[3].getLng()
        }
      },
      rotate: this.mapInstance.getRotation(),
      scale: this.mapInstance.getZoom(),
      skew: this.mapInstance.getPitch()
    }
  }

  mapUpdate (params) {
    if (!params) return
    params = Object.assign(params, params.setting || {})
    Object.keys(params).forEach(key => {
      if (watchKeys.indexOf(key) > -1) {
        this[`${key}Changed`](params[key])
      }
    })
  }

  latitudeChanged (val) {
    this.mapdata.latitude = val
    this.latitudeChangeFlag = true
    this.mapInstance.setCenter(new this.mapConstructor.LngLat(this.mapdata.longitude, this.mapdata.latitude))
  }

  longitudeChanged (val) {
    this.mapdata.longitude = val
    this.longitudeChangeFlag = true
    this.mapInstance.setCenter(new this.mapConstructor.LngLat(this.mapdata.longitude, this.mapdata.latitude))
  }

  scaleChanged (val) {
    if (val > this.mapdata.maxScale) {
      val = this.mapdata.maxScale
    } else if (val < this.mapdata.minScale) {
      val = this.mapdata.minScale
    }
    this.mapdata.scale = val
    this.mapInstance.setZoom(val)
  }

  skewChanged (val) {
    this.emitCustomEvent({
      event: 'custom_event_onMapTask',
      params: {
        eventName: 'regionchange',
        mapId: this.mapdata.mapId,
        causedBy: 'update',
        type: 'begin'
      }
    })
    this.mapdata.skew = val
    this.mapInstance.setPitch(val)
    const mapdata = this.getMapRegonData()
    this.emitCustomEvent({
      event: 'custom_event_onMapTask',
      params: {
        eventName: 'regionchange',
        mapId: this.mapdata.mapId,
        causedBy: 'update',
        type: 'end',
        ...mapdata
      }
    })
  }

  showCompassChanged (val) {
    this.mapdata.showCompass = val
  }

  showScaleChanged (val) {
    this.mapdata.showScale = val
    if (val) {
      this.scale.show()
    } else {
      this.scale.hide()
    }
  }

  enableZoomChanged (val) {
    this.mapdata.enableZoom = val
    this.mapInstance.setStatus({
      zoomEnable: val
    })
  }

  enableScrollChanged (val) {
    this.mapdata.enableScroll = val
    this.mapInstance.setStatus({
      dragEnable: val
    })
  }

  enableRotateChanged (val) {
    this.mapdata.enableRotate = val
    this.mapInstance.setStatus({
      rotateEnable: val
    })
  }

  enableSatelliteChanged (val) {
    this.mapdata.enableSatellite = val
    if (val) {
      this.mapInstance.add(this.satelliteLayer)
    } else {
      this.mapInstance.remove(this.satelliteLayer)
    }
  }

  enableTrafficChanged (val) {
    this.mapdata.enableTraffic = val
    if (val) {
      this.mapInstance.add(this.trafficLayer)
    } else {
      this.mapInstance.remove(this.trafficLayer)
    }
  }

  enableBuildingChanged (val) {
    this.mapdata.enableBuilding = (val === undefined || val === true)
    this.mapInstance.setFeatures(this.countFeatures())
  }

  doMapAction (data) {
    this[`mapAction${data.method}`] && this[`mapAction${data.method}`](data)
  }

  mapActionaddMarkers (data) {
    try {
      this.markersChanged(data.markers, data.clear)
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionfromScreenLocation (data) {
    try {
      const bounds = this.mapInstance.getBounds().bounds
      const nw = bounds[0]
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          latitude: nw.getLat(),
          longitude: nw.getLng()
        },
        noChangeEventName: true
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err, noChangeEventName: true })
    }
  }

  mapActiongetCenterLocation (data) {
    try {
      const lnglat = this.mapInstance.getCenter()
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          latitude: lnglat.getLat(),
          longitude: lnglat.getLng()
        }
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActiongetRegion (data) {
    try {
      const bounds = this.mapInstance.getBounds().bounds
      const ne = bounds[1]
      const sw = bounds[3]
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          northeast: {
            latitude: ne.getLat(),
            longitude: ne.getLng()
          },
          southwest: {
            latitude: sw.getLat(),
            longitude: sw.getLng()
          }
        }
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActiongetScale (data) {
    try {
      const scale = this.mapInstance.getZoom()
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          scale
        }
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActiongetSkew (data) {
    try {
      const skew = this.mapInstance.getPitch()
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          skew
        }
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActiongetRotate (data) {
    try {
      let rotate = this.mapInstance.getRotation()
      rotate += -(Math.floor((rotate) / 360) * 360)
      this.cbSuccess({
        data,
        cid: this.mapdata.cid,
        params: {
          rotate
        }
      })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionincludePoints(data) {
    try {
      const points = data.points
      if (points.length === 0) {
        throw new Error('参数错误')
      }

      const ne = points.reduce((previousItem, currentItem) => {
        return {
          longitude: Math.max(previousItem.longitude, currentItem.longitude),
          latitude: Math.max(previousItem.latitude, currentItem.latitude)
        }
      }, points[0])

      const sw = points.reduce((previousItem, currentItem) => {
        return {
          longitude: Math.min(previousItem.longitude, currentItem.longitude),
          latitude: Math.min(previousItem.latitude, currentItem.latitude)
        }
      }, points[0])
      const bounds = new this.mapConstructor.Bounds([ne.longitude, ne.latitude], [sw.longitude, sw.latitude])
      this.mapInstance.setBounds(bounds)
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActioninitMarkerCluster (data) {
    try {
      if(!this.makerCluster) {
        throw new Error('clusters plugin is initializing')
      }
      this.makerCluster.setGridSize(data.gridSize)
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionmoveAlong (data) {
    try {
      const { markerId, path = [], duration = 3000 } = data
      if (!markerId) {
        this.cbFail({ data, cid: this.mapdata.cid, err: 'markerId should be Number instead of Undefined' })
        return
      }
      const marker = this.markersInstence[markerId]
      if (marker) {
        const line = path.map(e => {
          return [e.longitude, e.latitude]
        })
        const distence = line.reduce((total, val, idx) => {
          if (idx === 0) return total
          const nowPosition = new this.mapConstructor.LngLat(line[idx - 1][0], line[idx - 1][1])
          const toPosition = new this.mapConstructor.LngLat(val[0], val[1])
          const dis = nowPosition.distance(toPosition)
          return (total + dis)
        }, 0)
        const speed = (distence / 1000) / ((duration / 1000) / 3600)
        marker.moveAlong(line, speed)
        setTimeout(() => {
          this.cbSuccess({
            data,
            cid: this.mapdata.cid
          })
        }, duration)
      } else {
        this.cbFail({ data, cid: this.mapdata.cid, err: ('can not find marker with id ' + markerId) })
        return
      }
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionmoveToLocation (data = { latitude: 39.92, longitude: 116.46 }) {
    try {
      this.moveToLocationChangeFlag = true
      this.mapInstance.setCenter(new this.mapConstructor.LngLat(data.longitude || 0, data.latitude || 0))
      this.cbSuccess({
        data,
        cid: this.mapdata.cid
      })
    } catch (err) {
      this.moveToLocationChangeFlag = false
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionopenMapApp (data) {
    try {
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionremoveMarkers (data) {
    try {
      this._removeMarkers(data.markerIds)
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionsetCenterOffset (data) {
    try {
      let offset = data.offset
      const regonData = this.initRegonData
      const min = 0.25
      const max = 0.75
      const range = max - min
      offset = offset.map(item => {
        if (item >= max) {
          item = max
        }
        if (item <= min) {
          item = min
        }
        return item
      })
      const xPercent = (range - (offset[0] - min)) / range
      const yPercent = ( (offset[1] - min)) / range

      const xPoint = (regonData.northeast.longitude - regonData.northwest.longitude) * xPercent + regonData.northwest.longitude
      const yPoint = (regonData.northeast.latitude - regonData.southeast.latitude) * yPercent + regonData.southeast.latitude

      if (this.initCenterOffset.length === 0 || this.initCenterOffset[0] !== offset[0] || this.initCenterOffset[1] !== offset[1]) {
        this.mapInstance.setCenter(new this.mapConstructor.LngLat(xPoint, yPoint))
        this.initCenterOffset = offset
      }
      this.cbSuccess({ data, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActionsetLocMarkerIcon (data) {
    this.cbFail({ data, cid: this.mapdata.cid, err: `${data.method}未实现` })
  }

  mapActiontoScreenLocation (data) {
    data.longitude = data.longitude ? data.longitude : 113.93041
    data.latitude = data.latitude ? data.latitude : 22.53332
    try {
      const lnglat = new this.mapConstructor.LngLat(data.longitude, data.latitude)
      const pixel = this.mapInstance.lngLatToContainer(lnglat)
      this.cbSuccess({ data, params: { ...pixel }, cid: this.mapdata.cid })
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  mapActiontranslateMarker (data) {
    try {
      let { markerId, destination, rotate, duration = 3000 } = data
      if (markerId === undefined) {
        this.cbFail({ data, cid: this.mapdata.cid, err: 'markerId should be Number instead of Undefined' })
        return
      }
      if (typeof markerId === 'object') {
        this.cbFail({ data, cid: this.mapdata.cid, err: 'error markerId' })
        return
      }
      if (!duration || isNaN(duration)) {
        duration = 3000
      }
      const marker = this.markersInstence[markerId]
      if (marker) {
        const nowPosition = marker.getPosition()
        const toPosition = new this.mapConstructor.LngLat(destination.longitude, destination.latitude)
        const distance = nowPosition.distance(toPosition)
        const speed = (distance / 1000) / ((duration / 1000) / 3600)
        marker.moveTo(toPosition, speed)
        !isNaN(rotate) && marker.setAngle(rotate)
        setTimeout(() => {
          this.cbSuccess({ data, cid: this.mapdata.cid })
        }, duration)
      } else {
        this.cbFail({ data, cid: this.mapdata.cid, err: ('can not find marker with id ' + markerId) })
        return
      }
    } catch (err) {
      this.cbFail({ data, cid: this.mapdata.cid, err })
    }
  }

  cbSuccess ({ data, cid, params = {}, noChangeEventName }) {
    const eventName = data.method
    this.FinChatJSBridge.publish('doneMapAction', {
      mapId: cid,
      callbackId: data.callbackId,
      eventName,
      res: {
        errMsg: `${eventName}:ok`,
        ...params
      }
    })
  }
  
  cbFail ({ data, cid, err, noChangeEventName }) {
    const eventName = data.method
    this.FinChatJSBridge.publish('doneMapAction', {
      mapId: cid,
      callbackId: data.callbackId,
      eventName,
      res: {
        errMsg: `${eventName}:fail ${err.toString()}`
      }
    })
  }
}

const watchKeys = [
  'latitude',
  'longitude',
  'scale',
  'markers',
  'polyline',
  'circles',
  'controls',
  'polygons',
  'rotate',
  'skew',
  'showCompass',
  'showScale',
  'enableZoom',
  'enableScroll',
  'enableRotate',
  'enableSatellite',
  'enableTraffic',
  'enablePoi',
  'enableBuilding'
]


